require_relative '../db/models/user'
require_relative '../db/models/credential'
require_relative 'tasks'

module Trackington
  class SprintRepository
    def initialize(project_id)
      @project_id = project_id
    end

    def create(data)
      data[:project_id] = @project_id
      sprint = Models::Sprint.new(data)
      sprint.is_active = true if current.nil?
      sprint.save
    end

    def current
      db_sprint = Models::Sprint.where(project_id: @project_id,
                                       is_active: true).first

      return db_sprint if db_sprint.nil?

      Sprint.new(db_sprint)
    end

    def get(sprint_id)
      db_sprint = Models::Sprint.find(sprint_id)
      Sprint.new(db_sprint)
    end

    def complete_sprint
      return if current.nil?

      current.tasks.move_to(backlog.id)
      current.end
    end

    def start_sprint
      project_sprints = Models::Sprint.where(project_id: @project_id)
      next_sprint_db = project_sprints.where('start_time > ?', Date.today).first

      return if next_sprint_db.nil?

      next_sprint = Sprint.new(next_sprint_db)

      backlog.tasks.move_to(next_sprint.id)
      next_sprint.start
    end

    def backlog
      query = Models::Sprint.where(project_id: @project_id, is_backlog: true)
      backlog = query.first
      Sprint.new(backlog)
    end
  end

  class Sprint
    attr_reader :id, :start_time, :end_time, :is_active, :is_backlog, :tasks

    def initialize(sprint)
      @id = sprint.id
      @start_time = sprint.start_time
      @end_time = sprint.end_time
      @is_active = sprint.is_active
      @is_backlog = sprint.is_backlog
      @project_id = sprint.project_id
      @tasks = TaskRepository.new(@id)
    end

    def start
      sprint = Models::Sprint.find(@id)
      sprint.is_active = true
      sprint.save
    end

    def end
      sprint = Models::Sprint.find(@id)
      sprint.is_active = false
      sprint.save
    end

    def add_task(user_id, data)
      check_user_role user_id
      data[:created_by] = user_id
      @tasks.add(data)
    end

    def all
      @tasks.all
    end

    private

    def check_user_role(user_id, role = 'spectator')
      repo = ProjectRepository.new
      project = repo.get(@project_id)
      project.user_is?(user_id, role)
    end
  end
end
