require 'rake'
require 'rake/tasklib'
require 'uri'
require 'active_record'

module Trackington
  class RakeTask < ::Rake::TaskLib
    include ::Rake::DSL if defined?(::Rake::DSL)

    def initialize
      namespace :trackington do
        namespace :db do
          task :migrate do
            establish_connection

            migrate
          end
        end
      end
    end

    private

    def establish_connection
      db_query = ENV['DB_QUERY']

      db = URI.parse(db_query)

      ActiveRecord::Base.establish_connection(
        adapter:  db.scheme == 'postgres' ? 'postgresql' : db.scheme,
        host:     db.host,
        username: db.user,
        password: db.password,
        database: db.path[1..-1],
        encoding: 'utf8')
    end

    def migrate
      migrations_path = File.expand_path('../../db/migrate', __FILE__)

      ActiveRecord::Migrator.migrate(migrations_path, nil)
    end
  end
end
